"""
834.5010.X220.A1.v2
Created 2023-05-19 10:17:35.966751
"""
from .base import *
from .common import *


class ST_LOOP_ST(Segment):
    """"""
    _segment_name = 'ST'
    
    st01: Annotated[D_143, Title(''), Usage('R'), Position(1), Enumerated(*['834'])]
    
    st02: Annotated[D_329, Title(''), Usage('R'), Position(2)]
    
    st03: Annotated[D_1705, Title(''), Usage('R'), Position(3), Enumerated(*['005010X220A1'])]


class HEADER_BGN(Segment):
    """"""
    _segment_name = 'BGN'
    
    bgn01: Annotated[D_353, Title(''), Usage('R'), Position(1), Enumerated(*['00', '15', '22'])]
    
    bgn02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    bgn03: Annotated[D_373, Title(''), Usage('R'), Position(3)]
    
    bgn04: Annotated[D_337, Title(''), Usage('R'), Position(4)]
    
    bgn05: Annotated[D_623, Title(''), Usage('S'), Position(5), Enumerated(*['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23', '24', 'AD', 'AS', 'AT', 'CD', 'CS', 'CT', 'ED', 'ES', 'ET', 'GM', 'HD', 'HS', 'HT', 'LT', 'MD', 'MS', 'MT', 'ND', 'NS', 'NT', 'PD', 'PS', 'PT', 'TD', 'TS', 'TT', 'UT'])]
    
    bgn06: Annotated[D_127, Title(''), Usage('S'), Position(6)]
    
    bgn07: Annotated[D_640, Title(''), Usage('N'), Position(7)]
    
    bgn08: Annotated[D_306, Title(''), Usage('R'), Position(8), Enumerated(*['2', '4', 'RX'])]
    
    bgn09: Annotated[D_786, Title(''), Usage('N'), Position(9)]


class HEADER_REF04(Composite):
    """"""
    pass


class HEADER_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['38'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class HEADER_DTP(Segment):
    """"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title(''), Usage('R'), Position(1), Enumerated(*['007', '090', '091', '303', '382', '388'])]
    
    dtp02: Annotated[D_1250, Title(''), Usage('R'), Position(2), Enumerated(*['D8'])]
    
    dtp03: Annotated[D_1251, Title(''), Usage('R'), Position(3)]


class HEADER_QTY03(Composite):
    """"""
    pass


class HEADER_QTY(Segment):
    """"""
    _segment_name = 'QTY'
    
    qty01: Annotated[D_673, Title(''), Usage('R'), Position(1), Enumerated(*['DT', 'ET', 'TO'])]
    
    qty02: Annotated[D_380, Title(''), Usage('R'), Position(2)]
    
    qty04: Annotated[D_61, Title(''), Usage('N'), Position(4)]


class L1000A_N1(Segment):
    """"""
    _segment_name = 'N1'
    
    n101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['P5'])]
    
    n102: Annotated[D_93, Title(''), Usage('S'), Position(2)]
    
    n103: Annotated[D_66, Title(''), Usage('R'), Position(3), Enumerated(*['24', '94', 'FI'])]
    
    n104: Annotated[D_67, Title(''), Usage('R'), Position(4)]
    
    n105: Annotated[D_706, Title(''), Usage('N'), Position(5)]
    
    n106: Annotated[D_98, Title(''), Usage('N'), Position(6)]


class L1000A(Loop):
    
    n1: Annotated[L1000A_N1, Title(''), Usage('R'), Position(700), Syntax(['R0203', 'P0304']), Required(True)]


class L1000B_N1(Segment):
    """"""
    _segment_name = 'N1'
    
    n101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['IN'])]
    
    n102: Annotated[D_93, Title(''), Usage('S'), Position(2)]
    
    n103: Annotated[D_66, Title(''), Usage('R'), Position(3), Enumerated(*['94', 'FI', 'XV'])]
    
    n104: Annotated[D_67, Title(''), Usage('R'), Position(4)]
    
    n105: Annotated[D_706, Title(''), Usage('N'), Position(5)]
    
    n106: Annotated[D_98, Title(''), Usage('N'), Position(6)]


class L1000B(Loop):
    
    n1: Annotated[L1000B_N1, Title(''), Usage('R'), Position(700), Syntax(['R0203', 'P0304']), Required(True)]


class L1000C_N1(Segment):
    """"""
    _segment_name = 'N1'
    
    n101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['BO', 'TV'])]
    
    n102: Annotated[D_93, Title(''), Usage('R'), Position(2)]
    
    n103: Annotated[D_66, Title(''), Usage('R'), Position(3), Enumerated(*['94', 'FI', 'XV'])]
    
    n104: Annotated[D_67, Title(''), Usage('R'), Position(4)]
    
    n105: Annotated[D_706, Title(''), Usage('N'), Position(5)]
    
    n106: Annotated[D_98, Title(''), Usage('N'), Position(6)]


class L1100C_ACT(Segment):
    """"""
    _segment_name = 'ACT'
    
    act01: Annotated[D_508, Title(''), Usage('R'), Position(1)]
    
    act02: Annotated[D_93, Title(''), Usage('N'), Position(2)]
    
    act03: Annotated[D_66, Title(''), Usage('N'), Position(3)]
    
    act04: Annotated[D_67, Title(''), Usage('N'), Position(4)]
    
    act05: Annotated[D_569, Title(''), Usage('N'), Position(5)]
    
    act06: Annotated[D_508, Title(''), Usage('S'), Position(6)]
    
    act07: Annotated[D_352, Title(''), Usage('N'), Position(7)]
    
    act08: Annotated[D_107, Title(''), Usage('N'), Position(8)]
    
    act09: Annotated[D_1216, Title(''), Usage('N'), Position(9)]


class L1100C(Loop):
    
    act: Annotated[L1100C_ACT, Title(''), Usage('R'), Position(1200), Syntax(['P0304', 'C0506', 'C0705']), Required(True)]


class L1000C(Loop):
    
    n1: Annotated[L1000C_N1, Title(''), Usage('R'), Position(700), Syntax(['R0203', 'P0304']), Required(True)]
    ItemL1100C: TypeAlias = Annotated[L1100C, Title(''), Usage('R'), Position(3600), Required(True)]
    l1100c: Annotated[list[ItemL1100C], MinItems(1)]


class HEADER(Loop):
    
    bgn: Annotated[HEADER_BGN, Title(''), Usage('R'), Position(200), Syntax(['C0504']), Required(True)]
    
    ref: Annotated[HEADER_REF, Title(''), Usage('S'), Position(300), Syntax(['R0203']), Required(True)]
    ItemDtp: TypeAlias = Annotated[HEADER_DTP, Title(''), Usage('S'), Position(400), Required(True)]
    dtp: Annotated[list[ItemDtp], MinItems(1)]
    ItemQty: TypeAlias = Annotated[HEADER_QTY, Title(''), Usage('S'), Position(600), Syntax(['E0204', 'R0204']), Required(True)]
    qty: Annotated[list[ItemQty], MaxItems(3)]
    ItemL1000A: TypeAlias = Annotated[L1000A, Title(''), Usage('R'), Position(700), Required(True)]
    l1000a: Annotated[list[ItemL1000A], MinItems(1)]
    ItemL1000B: TypeAlias = Annotated[L1000B, Title(''), Usage('R'), Position(1900), Required(True)]
    l1000b: Annotated[list[ItemL1000B], MinItems(1)]
    ItemL1000C: TypeAlias = Annotated[L1000C, Title(''), Usage('R'), Position(3100), Required(True)]
    l1000c: Annotated[list[ItemL1000C], MinItems(2)]


class L2000_INS06(Composite):
    """"""
    
    ins06_01: Annotated[D_1218, Title(''), Usage('R'), Position(1), Enumerated(*['A', 'B', 'C', 'D', 'E'])]
    
    ins06_02: Annotated[D_1701, Title(''), Usage('S'), Position(2), Enumerated(*['0', '1', '2'])]
    
    ins06_03: Annotated[D_1701, Title(''), Usage('N'), Position(3)]
    
    ins06_04: Annotated[D_1701, Title(''), Usage('N'), Position(4)]


class L2000_INS(Segment):
    """"""
    _segment_name = 'INS'
    
    ins01: Annotated[D_1073, Title(''), Usage('R'), Position(1), Enumerated(*['N', 'Y'])]
    
    ins02: Annotated[D_1069, Title(''), Usage('R'), Position(2), Enumerated(*['01', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '23', '24', '25', '26', '31', '38', '53', '60', 'D2', 'G8', 'G9'])]
    
    ins03: Annotated[D_875, Title(''), Usage('R'), Position(3), Enumerated(*['001', '021', '024', '025', '030'])]
    
    ins04: Annotated[D_1203, Title(''), Usage('S'), Position(4), Enumerated(*['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '14', '15', '16', '17', '18', '20', '21', '22', '25', '26', '27', '28', '29', '31', '32', '33', '37', '38', '39', '40', '41', '43', '59', 'AA', 'AB', 'AC', 'AD', 'AE', 'AF', 'AG', 'AH', 'AI', 'AJ', 'AL', 'EC', 'XN', 'XT'])]
    
    ins05: Annotated[D_1216, Title(''), Usage('R'), Position(5), Enumerated(*['A', 'C', 'S', 'T'])]
    
    ins06: Annotated[L2000_INS06, Title(''), Usage('S'), Position(6), Required(True)]
    
    ins07: Annotated[D_1219, Title(''), Usage('S'), Position(7), Enumerated(*['1', '10', '2', '3', '4', '5', '6', '7', '8', '9', 'ZZ'])]
    
    ins08: Annotated[D_584, Title(''), Usage('S'), Position(8), Enumerated(*['AC', 'AO', 'AU', 'FT', 'L1', 'PT', 'RT', 'TE'])]
    
    ins09: Annotated[D_1220, Title(''), Usage('S'), Position(9), Enumerated(*['F', 'N', 'P'])]
    
    ins10: Annotated[D_1073, Title(''), Usage('S'), Position(10), Enumerated(*['N', 'Y'])]
    
    ins11: Annotated[D_1250, Title(''), Usage('S'), Position(11), Enumerated(*['D8'])]
    
    ins12: Annotated[D_1251, Title(''), Usage('S'), Position(12)]
    
    ins13: Annotated[D_1165, Title(''), Usage('S'), Position(13), Enumerated(*['R', 'U'])]
    
    ins14: Annotated[D_19, Title(''), Usage('N'), Position(14)]
    
    ins15: Annotated[D_156, Title(''), Usage('N'), Position(15)]
    
    ins16: Annotated[D_26, Title(''), Usage('N'), Position(16)]
    
    ins17: Annotated[D_1470, Title(''), Usage('S'), Position(17)]


class L2000_REF04(Composite):
    """"""
    pass


class L2000_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['0F'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L2000_REF04(Composite):
    """"""
    pass


class L2000_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['1L'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L2000_REF04(Composite):
    """"""
    pass


class L2000_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['17', '23', '3H', '4A', '6O', 'ABB', 'D3', 'DX', 'F6', 'P5', 'Q4', 'QQ', 'ZZ'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L2000_DTP(Segment):
    """"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title(''), Usage('R'), Position(1), Enumerated(*['050', '286', '296', '297', '300', '301', '303', '336', '337', '338', '339', '340', '341', '350', '351', '356', '357', '383', '385', '386', '393', '394', '473', '474'])]
    
    dtp02: Annotated[D_1250, Title(''), Usage('R'), Position(2), Enumerated(*['D8'])]
    
    dtp03: Annotated[D_1251, Title(''), Usage('R'), Position(3)]


class L2100A_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['74', 'IL'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['1'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('S'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('S'), Position(8), Enumerated(*['34', 'ZZ'])]
    
    nm109: Annotated[D_67, Title(''), Usage('S'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100A_PER(Segment):
    """"""
    _segment_name = 'PER'
    
    per01: Annotated[D_366, Title(''), Usage('R'), Position(1), Enumerated(*['IP'])]
    
    per02: Annotated[D_93, Title(''), Usage('N'), Position(2)]
    
    per03: Annotated[D_365, Title(''), Usage('R'), Position(3), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'HP', 'TE', 'WP'])]
    
    per04: Annotated[D_364, Title(''), Usage('R'), Position(4)]
    
    per05: Annotated[D_365, Title(''), Usage('S'), Position(5), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'HP', 'TE', 'WP'])]
    
    per06: Annotated[D_364, Title(''), Usage('S'), Position(6)]
    
    per07: Annotated[D_365, Title(''), Usage('S'), Position(7), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'HP', 'TE', 'WP'])]
    
    per08: Annotated[D_364, Title(''), Usage('S'), Position(8)]
    
    per09: Annotated[D_443, Title(''), Usage('N'), Position(9)]


class L2100A_N3(Segment):
    """"""
    _segment_name = 'N3'
    
    n301: Annotated[D_166, Title(''), Usage('R'), Position(1)]
    
    n302: Annotated[D_166, Title(''), Usage('S'), Position(2)]


class L2100A_N4(Segment):
    """"""
    _segment_name = 'N4'
    
    n401: Annotated[D_19, Title(''), Usage('R'), Position(1)]
    
    n402: Annotated[D_156, Title(''), Usage('S'), Position(2), Enumerated(*states)]
    
    n403: Annotated[D_116, Title(''), Usage('S'), Position(3)]
    
    n404: Annotated[D_26, Title(''), Usage('S'), Position(4), Enumerated(*country)]
    
    n405: Annotated[D_309, Title(''), Usage('S'), Position(5), Enumerated(*['60', 'CY'])]
    
    n406: Annotated[D_310, Title(''), Usage('S'), Position(6)]
    
    n407: Annotated[D_1715, Title(''), Usage('S'), Position(7)]


class L2100A_DMG05(Composite):
    """"""
    
    dmg05_01: Annotated[D_1109, Title(''), Usage('S'), Position(1), Enumerated(*['7', '8', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'N', 'O', 'P', 'Z'])]
    
    dmg05_02: Annotated[D_1270, Title(''), Usage('S'), Position(2), Enumerated(*['RET'])]
    
    dmg05_03: Annotated[D_1271, Title(''), Usage('S'), Position(3)]


class L2100A_DMG(Segment):
    """"""
    _segment_name = 'DMG'
    
    dmg01: Annotated[D_1250, Title(''), Usage('R'), Position(1), Enumerated(*['D8'])]
    
    dmg02: Annotated[D_1251, Title(''), Usage('R'), Position(2)]
    
    dmg03: Annotated[D_1068, Title(''), Usage('R'), Position(3), Enumerated(*['F', 'M', 'U'])]
    
    dmg04: Annotated[D_1067, Title(''), Usage('S'), Position(4), Enumerated(*['B', 'D', 'I', 'M', 'R', 'S', 'U', 'W', 'X'])]
    
    dmg05: Annotated[L2100A_DMG05, Title(''), Usage('S'), Position(5)]
    
    dmg06: Annotated[D_1066, Title(''), Usage('S'), Position(6), Enumerated(*['1', '2', '3', '4', '5', '6', '7'])]
    
    dmg07: Annotated[D_26, Title(''), Usage('N'), Position(7)]
    
    dmg08: Annotated[D_659, Title(''), Usage('N'), Position(8)]
    
    dmg09: Annotated[D_380, Title(''), Usage('N'), Position(9)]
    
    dmg10: Annotated[D_1270, Title(''), Usage('S'), Position(10), Enumerated(*['REC'])]
    
    dmg11: Annotated[D_1271, Title(''), Usage('S'), Position(11)]


class L2100A_EC(Segment):
    """"""
    _segment_name = 'EC'
    
    ec01: Annotated[D_1176, Title(''), Usage('R'), Position(1), Enumerated(*['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12', '17', '18', '19', '20', '21', '22', '23'])]
    
    ec02: Annotated[D_1176, Title(''), Usage('S'), Position(2), Enumerated(*['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12', '17', '18', '19', '20', '21', '22', '23'])]
    
    ec03: Annotated[D_1176, Title(''), Usage('S'), Position(3), Enumerated(*['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12', '17', '18', '19', '20', '21', '22', '23'])]
    
    ec04: Annotated[D_954, Title(''), Usage('N'), Position(4)]
    
    ec05: Annotated[D_1201, Title(''), Usage('N'), Position(5)]
    
    ec06: Annotated[D_1149, Title(''), Usage('N'), Position(6)]


class L2100A_ICM(Segment):
    """"""
    _segment_name = 'ICM'
    
    icm01: Annotated[D_594, Title(''), Usage('R'), Position(1), Enumerated(*['1', '2', '3', '4', '6', '7', '8', '9', 'B', 'C', 'H', 'Q', 'S', 'U'])]
    
    icm02: Annotated[D_782, Title(''), Usage('R'), Position(2)]
    
    icm03: Annotated[D_380, Title(''), Usage('S'), Position(3)]
    
    icm04: Annotated[D_310, Title(''), Usage('S'), Position(4)]
    
    icm05: Annotated[D_1214, Title(''), Usage('S'), Position(5)]
    
    icm06: Annotated[D_100, Title(''), Usage('N'), Position(6)]


class L2100A_AMT(Segment):
    """"""
    _segment_name = 'AMT'
    
    amt01: Annotated[D_522, Title(''), Usage('R'), Position(1), Enumerated(*['B9', 'C1', 'D2', 'EBA', 'FK', 'P3', 'R'])]
    
    amt02: Annotated[D_782, Title(''), Usage('R'), Position(2)]
    
    amt03: Annotated[D_478, Title(''), Usage('N'), Position(3)]


class L2100A_HLH(Segment):
    """"""
    _segment_name = 'HLH'
    
    hlh01: Annotated[D_1212, Title(''), Usage('R'), Position(1), Enumerated(*['N', 'S', 'T', 'U', 'X'])]
    
    hlh02: Annotated[D_65, Title(''), Usage('S'), Position(2)]
    
    hlh03: Annotated[D_81, Title(''), Usage('S'), Position(3)]
    
    hlh04: Annotated[D_81, Title(''), Usage('N'), Position(4)]
    
    hlh05: Annotated[D_352, Title(''), Usage('N'), Position(5)]
    
    hlh06: Annotated[D_1213, Title(''), Usage('N'), Position(6)]
    
    hlh07: Annotated[D_352, Title(''), Usage('N'), Position(7)]


class L2100A_LUI(Segment):
    """"""
    _segment_name = 'LUI'
    
    lui01: Annotated[D_66, Title(''), Usage('S'), Position(1), Enumerated(*['LD', 'LE'])]
    
    lui02: Annotated[D_67, Title(''), Usage('S'), Position(2)]
    
    lui03: Annotated[D_352, Title(''), Usage('S'), Position(3)]
    
    lui04: Annotated[D_1303, Title(''), Usage('S'), Position(4), Enumerated(*['5', '6', '7', '8'])]
    
    lui05: Annotated[D_1476, Title(''), Usage('N'), Position(5)]


class L2100A(Loop):
    
    nm1: Annotated[L2100A_NM1, Title(''), Usage('R'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    per: Annotated[L2100A_PER, Title(''), Usage('S'), Position(400), Syntax(['P0304', 'P0506', 'P0708']), Required(True)]
    
    n3: Annotated[L2100A_N3, Title(''), Usage('S'), Position(500), Required(True)]
    
    n4: Annotated[L2100A_N4, Title(''), Usage('S'), Position(600), Syntax(['E0207', 'C0605', 'C0704']), Required(True)]
    
    dmg: Annotated[L2100A_DMG, Title(''), Usage('S'), Position(800), Syntax(['P0102', 'P1011', 'C1105']), Required(True)]
    ItemEc: TypeAlias = Annotated[L2100A_EC, Title(''), Usage('S'), Position(1000), Required(True)]
    ec: Annotated[list[ItemEc], MinItems(1)]
    
    icm: Annotated[L2100A_ICM, Title(''), Usage('S'), Position(1100), Required(True)]
    ItemAmt: TypeAlias = Annotated[L2100A_AMT, Title(''), Usage('S'), Position(1200), Required(True)]
    amt: Annotated[list[ItemAmt], MaxItems(7)]
    
    hlh: Annotated[L2100A_HLH, Title(''), Usage('S'), Position(1300), Required(True)]
    ItemLui: TypeAlias = Annotated[L2100A_LUI, Title(''), Usage('S'), Position(1500), Syntax(['P0102', 'L040203'])]
    lui: Annotated[list[ItemLui], MinItems(1)]


class L2100B_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['70'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['1'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('S'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('S'), Position(8), Enumerated(*['34', 'ZZ'])]
    
    nm109: Annotated[D_67, Title(''), Usage('S'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100B_DMG05(Composite):
    """"""
    
    dmg05_01: Annotated[D_1109, Title(''), Usage('S'), Position(1), Enumerated(*['7', '8', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'N', 'O', 'P', 'Z'])]
    
    dmg05_02: Annotated[D_1270, Title(''), Usage('S'), Position(2), Enumerated(*['RET'])]
    
    dmg05_03: Annotated[D_1271, Title(''), Usage('S'), Position(3)]


class L2100B_DMG(Segment):
    """"""
    _segment_name = 'DMG'
    
    dmg01: Annotated[D_1250, Title(''), Usage('S'), Position(1), Enumerated(*['D8'])]
    
    dmg02: Annotated[D_1251, Title(''), Usage('S'), Position(2)]
    
    dmg03: Annotated[D_1068, Title(''), Usage('S'), Position(3), Enumerated(*['F', 'M', 'U'])]
    
    dmg04: Annotated[D_1067, Title(''), Usage('S'), Position(4), Enumerated(*['B', 'D', 'I', 'M', 'R', 'S', 'U', 'W', 'X'])]
    
    dmg05: Annotated[L2100B_DMG05, Title(''), Usage('S'), Position(5)]
    
    dmg06: Annotated[D_1066, Title(''), Usage('S'), Position(6), Enumerated(*['1', '2', '3', '4', '5', '6', '7'])]
    
    dmg07: Annotated[D_26, Title(''), Usage('N'), Position(7)]
    
    dmg08: Annotated[D_659, Title(''), Usage('N'), Position(8)]
    
    dmg09: Annotated[D_380, Title(''), Usage('N'), Position(9)]
    
    dmg10: Annotated[D_1270, Title(''), Usage('S'), Position(10), Enumerated(*['REC'])]
    
    dmg11: Annotated[D_1271, Title(''), Usage('S'), Position(11)]


class L2100B(Loop):
    
    nm1: Annotated[L2100B_NM1, Title(''), Usage('R'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    dmg: Annotated[L2100B_DMG, Title(''), Usage('S'), Position(800), Syntax(['P0102', 'P1011', 'C1105'])]


class L2100C_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['31'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['1'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('N'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('N'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('N'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('N'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('N'), Position(8)]
    
    nm109: Annotated[D_67, Title(''), Usage('N'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100C_N3(Segment):
    """"""
    _segment_name = 'N3'
    
    n301: Annotated[D_166, Title(''), Usage('R'), Position(1)]
    
    n302: Annotated[D_166, Title(''), Usage('S'), Position(2)]


class L2100C_N4(Segment):
    """"""
    _segment_name = 'N4'
    
    n401: Annotated[D_19, Title(''), Usage('R'), Position(1)]
    
    n402: Annotated[D_156, Title(''), Usage('S'), Position(2), Enumerated(*states)]
    
    n403: Annotated[D_116, Title(''), Usage('S'), Position(3)]
    
    n404: Annotated[D_26, Title(''), Usage('S'), Position(4), Enumerated(*country)]
    
    n405: Annotated[D_309, Title(''), Usage('N'), Position(5)]
    
    n406: Annotated[D_310, Title(''), Usage('N'), Position(6)]
    
    n407: Annotated[D_1715, Title(''), Usage('S'), Position(7)]


class L2100C(Loop):
    
    nm1: Annotated[L2100C_NM1, Title(''), Usage('R'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    n3: Annotated[L2100C_N3, Title(''), Usage('R'), Position(500), Required(True)]
    
    n4: Annotated[L2100C_N4, Title(''), Usage('R'), Position(600), Syntax(['E0207', 'C0605', 'C0704']), Required(True)]


class L2100D_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['36'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['1', '2'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('S'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('S'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('S'), Position(8), Enumerated(*['24', '34'])]
    
    nm109: Annotated[D_67, Title(''), Usage('S'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100D_PER(Segment):
    """"""
    _segment_name = 'PER'
    
    per01: Annotated[D_366, Title(''), Usage('R'), Position(1), Enumerated(*['EP'])]
    
    per02: Annotated[D_93, Title(''), Usage('S'), Position(2)]
    
    per03: Annotated[D_365, Title(''), Usage('R'), Position(3), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'TE'])]
    
    per04: Annotated[D_364, Title(''), Usage('R'), Position(4)]
    
    per05: Annotated[D_365, Title(''), Usage('S'), Position(5), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'TE'])]
    
    per06: Annotated[D_364, Title(''), Usage('S'), Position(6)]
    
    per07: Annotated[D_365, Title(''), Usage('S'), Position(7), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'TE'])]
    
    per08: Annotated[D_364, Title(''), Usage('S'), Position(8)]
    
    per09: Annotated[D_443, Title(''), Usage('N'), Position(9)]


class L2100D_N3(Segment):
    """"""
    _segment_name = 'N3'
    
    n301: Annotated[D_166, Title(''), Usage('R'), Position(1)]
    
    n302: Annotated[D_166, Title(''), Usage('S'), Position(2)]


class L2100D_N4(Segment):
    """"""
    _segment_name = 'N4'
    
    n401: Annotated[D_19, Title(''), Usage('R'), Position(1)]
    
    n402: Annotated[D_156, Title(''), Usage('S'), Position(2), Enumerated(*states)]
    
    n403: Annotated[D_116, Title(''), Usage('S'), Position(3)]
    
    n404: Annotated[D_26, Title(''), Usage('S'), Position(4), Enumerated(*country)]
    
    n405: Annotated[D_309, Title(''), Usage('N'), Position(5)]
    
    n406: Annotated[D_310, Title(''), Usage('N'), Position(6)]
    
    n407: Annotated[D_1715, Title(''), Usage('S'), Position(7)]


class L2100D(Loop):
    
    nm1: Annotated[L2100D_NM1, Title(''), Usage('R'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    per: Annotated[L2100D_PER, Title(''), Usage('S'), Position(400), Syntax(['P0304', 'P0506', 'P0708']), Required(True)]
    
    n3: Annotated[L2100D_N3, Title(''), Usage('S'), Position(500), Required(True)]
    
    n4: Annotated[L2100D_N4, Title(''), Usage('S'), Position(600), Syntax(['E0207', 'C0605', 'C0704']), Required(True)]


class L2100E_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['M8'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['2'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('N'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('N'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('N'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('N'), Position(8)]
    
    nm109: Annotated[D_67, Title(''), Usage('N'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100E_PER(Segment):
    """"""
    _segment_name = 'PER'
    
    per01: Annotated[D_366, Title(''), Usage('R'), Position(1), Enumerated(*['SK'])]
    
    per02: Annotated[D_93, Title(''), Usage('S'), Position(2)]
    
    per03: Annotated[D_365, Title(''), Usage('R'), Position(3), Enumerated(*['EM', 'EX', 'FX', 'TE'])]
    
    per04: Annotated[D_364, Title(''), Usage('R'), Position(4)]
    
    per05: Annotated[D_365, Title(''), Usage('S'), Position(5), Enumerated(*['EM', 'EX', 'FX', 'TE'])]
    
    per06: Annotated[D_364, Title(''), Usage('S'), Position(6)]
    
    per07: Annotated[D_365, Title(''), Usage('S'), Position(7), Enumerated(*['EM', 'EX', 'FX', 'TE'])]
    
    per08: Annotated[D_364, Title(''), Usage('S'), Position(8)]
    
    per09: Annotated[D_443, Title(''), Usage('N'), Position(9)]


class L2100E_N3(Segment):
    """"""
    _segment_name = 'N3'
    
    n301: Annotated[D_166, Title(''), Usage('R'), Position(1)]
    
    n302: Annotated[D_166, Title(''), Usage('S'), Position(2)]


class L2100E_N4(Segment):
    """"""
    _segment_name = 'N4'
    
    n401: Annotated[D_19, Title(''), Usage('R'), Position(1)]
    
    n402: Annotated[D_156, Title(''), Usage('S'), Position(2), Enumerated(*states)]
    
    n403: Annotated[D_116, Title(''), Usage('S'), Position(3)]
    
    n404: Annotated[D_26, Title(''), Usage('S'), Position(4), Enumerated(*country)]
    
    n405: Annotated[D_309, Title(''), Usage('N'), Position(5)]
    
    n406: Annotated[D_310, Title(''), Usage('N'), Position(6)]
    
    n407: Annotated[D_1715, Title(''), Usage('S'), Position(7)]


class L2100E(Loop):
    
    nm1: Annotated[L2100E_NM1, Title(''), Usage('R'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    per: Annotated[L2100E_PER, Title(''), Usage('S'), Position(400), Syntax(['P0304', 'P0506', 'P0708']), Required(True)]
    
    n3: Annotated[L2100E_N3, Title(''), Usage('S'), Position(500), Required(True)]
    
    n4: Annotated[L2100E_N4, Title(''), Usage('S'), Position(600), Syntax(['E0207', 'C0605', 'C0704']), Required(True)]


class L2100F_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['S3'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['1'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('R'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('S'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('S'), Position(8), Enumerated(*['34', 'ZZ'])]
    
    nm109: Annotated[D_67, Title(''), Usage('S'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100F_PER(Segment):
    """"""
    _segment_name = 'PER'
    
    per01: Annotated[D_366, Title(''), Usage('R'), Position(1), Enumerated(*['PQ'])]
    
    per02: Annotated[D_93, Title(''), Usage('N'), Position(2)]
    
    per03: Annotated[D_365, Title(''), Usage('R'), Position(3), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'HP', 'TE', 'WP'])]
    
    per04: Annotated[D_364, Title(''), Usage('R'), Position(4)]
    
    per05: Annotated[D_365, Title(''), Usage('S'), Position(5), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'HP', 'TE', 'WP'])]
    
    per06: Annotated[D_364, Title(''), Usage('S'), Position(6)]
    
    per07: Annotated[D_365, Title(''), Usage('S'), Position(7), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'HP', 'TE', 'WP'])]
    
    per08: Annotated[D_364, Title(''), Usage('S'), Position(8)]
    
    per09: Annotated[D_443, Title(''), Usage('N'), Position(9)]


class L2100F_N3(Segment):
    """"""
    _segment_name = 'N3'
    
    n301: Annotated[D_166, Title(''), Usage('R'), Position(1)]
    
    n302: Annotated[D_166, Title(''), Usage('S'), Position(2)]


class L2100F_N4(Segment):
    """"""
    _segment_name = 'N4'
    
    n401: Annotated[D_19, Title(''), Usage('R'), Position(1)]
    
    n402: Annotated[D_156, Title(''), Usage('S'), Position(2), Enumerated(*states)]
    
    n403: Annotated[D_116, Title(''), Usage('S'), Position(3)]
    
    n404: Annotated[D_26, Title(''), Usage('S'), Position(4), Enumerated(*country)]
    
    n405: Annotated[D_309, Title(''), Usage('N'), Position(5)]
    
    n406: Annotated[D_310, Title(''), Usage('N'), Position(6)]
    
    n407: Annotated[D_1715, Title(''), Usage('S'), Position(7)]


class L2100F(Loop):
    
    nm1: Annotated[L2100F_NM1, Title(''), Usage('R'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    per: Annotated[L2100F_PER, Title(''), Usage('S'), Position(400), Syntax(['P0304', 'P0506', 'P0708']), Required(True)]
    
    n3: Annotated[L2100F_N3, Title(''), Usage('S'), Position(500), Required(True)]
    
    n4: Annotated[L2100F_N4, Title(''), Usage('S'), Position(600), Syntax(['E0207', 'C0605', 'C0704']), Required(True)]


class L2100G_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['6Y', '9K', 'E1', 'EI', 'EXS', 'GB', 'GD', 'J6', 'LR', 'QD', 'S1', 'TZ', 'X4'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['1'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('R'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('S'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('S'), Position(8), Enumerated(*['34', 'ZZ'])]
    
    nm109: Annotated[D_67, Title(''), Usage('S'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100G_PER(Segment):
    """"""
    _segment_name = 'PER'
    
    per01: Annotated[D_366, Title(''), Usage('R'), Position(1), Enumerated(*['RP'])]
    
    per02: Annotated[D_93, Title(''), Usage('N'), Position(2)]
    
    per03: Annotated[D_365, Title(''), Usage('R'), Position(3), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'HP', 'TE', 'WP'])]
    
    per04: Annotated[D_364, Title(''), Usage('R'), Position(4)]
    
    per05: Annotated[D_365, Title(''), Usage('S'), Position(5), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'HP', 'TE', 'WP'])]
    
    per06: Annotated[D_364, Title(''), Usage('S'), Position(6)]
    
    per07: Annotated[D_365, Title(''), Usage('S'), Position(7), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'HP', 'TE', 'WP'])]
    
    per08: Annotated[D_364, Title(''), Usage('S'), Position(8)]
    
    per09: Annotated[D_443, Title(''), Usage('N'), Position(9)]


class L2100G_N3(Segment):
    """"""
    _segment_name = 'N3'
    
    n301: Annotated[D_166, Title(''), Usage('R'), Position(1)]
    
    n302: Annotated[D_166, Title(''), Usage('S'), Position(2)]


class L2100G_N4(Segment):
    """"""
    _segment_name = 'N4'
    
    n401: Annotated[D_19, Title(''), Usage('R'), Position(1)]
    
    n402: Annotated[D_156, Title(''), Usage('S'), Position(2), Enumerated(*states)]
    
    n403: Annotated[D_116, Title(''), Usage('S'), Position(3)]
    
    n404: Annotated[D_26, Title(''), Usage('S'), Position(4), Enumerated(*country)]
    
    n405: Annotated[D_309, Title(''), Usage('N'), Position(5)]
    
    n406: Annotated[D_310, Title(''), Usage('N'), Position(6)]
    
    n407: Annotated[D_1715, Title(''), Usage('S'), Position(7)]


class L2100G(Loop):
    
    nm1: Annotated[L2100G_NM1, Title(''), Usage('R'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    per: Annotated[L2100G_PER, Title(''), Usage('S'), Position(400), Syntax(['P0304', 'P0506', 'P0708']), Required(True)]
    
    n3: Annotated[L2100G_N3, Title(''), Usage('S'), Position(500), Required(True)]
    
    n4: Annotated[L2100G_N4, Title(''), Usage('S'), Position(600), Syntax(['E0207', 'C0605', 'C0704']), Required(True)]


class L2100H_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['45'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['1'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('S'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('S'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('N'), Position(8)]
    
    nm109: Annotated[D_67, Title(''), Usage('N'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2100H_N3(Segment):
    """"""
    _segment_name = 'N3'
    
    n301: Annotated[D_166, Title(''), Usage('R'), Position(1)]
    
    n302: Annotated[D_166, Title(''), Usage('S'), Position(2)]


class L2100H_N4(Segment):
    """"""
    _segment_name = 'N4'
    
    n401: Annotated[D_19, Title(''), Usage('R'), Position(1)]
    
    n402: Annotated[D_156, Title(''), Usage('S'), Position(2), Enumerated(*states)]
    
    n403: Annotated[D_116, Title(''), Usage('S'), Position(3)]
    
    n404: Annotated[D_26, Title(''), Usage('S'), Position(4), Enumerated(*country)]
    
    n405: Annotated[D_309, Title(''), Usage('N'), Position(5)]
    
    n406: Annotated[D_310, Title(''), Usage('N'), Position(6)]
    
    n407: Annotated[D_1715, Title(''), Usage('S'), Position(7)]


class L2100H(Loop):
    
    nm1: Annotated[L2100H_NM1, Title(''), Usage('R'), Position(300), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    n3: Annotated[L2100H_N3, Title(''), Usage('S'), Position(500), Required(True)]
    
    n4: Annotated[L2100H_N4, Title(''), Usage('S'), Position(600), Syntax(['E0207', 'C0605', 'C0704']), Required(True)]


class L2200_DSB(Segment):
    """"""
    _segment_name = 'DSB'
    
    dsb01: Annotated[D_1146, Title(''), Usage('R'), Position(1), Enumerated(*['1', '2', '3', '4'])]
    
    dsb02: Annotated[D_380, Title(''), Usage('N'), Position(2)]
    
    dsb03: Annotated[D_1149, Title(''), Usage('N'), Position(3)]
    
    dsb04: Annotated[D_1154, Title(''), Usage('N'), Position(4)]
    
    dsb05: Annotated[D_1161, Title(''), Usage('N'), Position(5)]
    
    dsb06: Annotated[D_782, Title(''), Usage('N'), Position(6)]
    
    dsb07: Annotated[D_235, Title(''), Usage('S'), Position(7), Enumerated(*['DX', 'ZZ'])]
    
    dsb08: Annotated[D_1137, Title(''), Usage('S'), Position(8)]


class L2200_DTP(Segment):
    """"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title(''), Usage('R'), Position(1), Enumerated(*['360', '361'])]
    
    dtp02: Annotated[D_1250, Title(''), Usage('R'), Position(2), Enumerated(*['D8'])]
    
    dtp03: Annotated[D_1251, Title(''), Usage('R'), Position(3)]


class L2200(Loop):
    
    dsb: Annotated[L2200_DSB, Title(''), Usage('R'), Position(2000), Syntax(['P0708']), Required(True)]
    ItemDtp: TypeAlias = Annotated[L2200_DTP, Title(''), Usage('S'), Position(2100), Required(True)]
    dtp: Annotated[list[ItemDtp], MaxItems(2)]


class L2300_HD(Segment):
    """"""
    _segment_name = 'HD'
    
    hd01: Annotated[D_875, Title(''), Usage('R'), Position(1), Enumerated(*['001', '002', '021', '024', '025', '026', '030', '032'])]
    
    hd02: Annotated[D_1203, Title(''), Usage('N'), Position(2)]
    
    hd03: Annotated[D_1205, Title(''), Usage('R'), Position(3), Enumerated(*['AG', 'AH', 'AJ', 'AK', 'DCP', 'DEN', 'EPO', 'FAC', 'HE', 'HLT', 'HMO', 'LTC', 'LTD', 'MM', 'MOD', 'PDG', 'POS', 'PPO', 'PRA', 'STD', 'UR', 'VIS'])]
    
    hd04: Annotated[D_1204, Title(''), Usage('S'), Position(4)]
    
    hd05: Annotated[D_1207, Title(''), Usage('S'), Position(5), Enumerated(*['CHD', 'DEP', 'E1D', 'E2D', 'E3D', 'E5D', 'E6D', 'E7D', 'E8D', 'E9D', 'ECH', 'EMP', 'ESP', 'FAM', 'IND', 'SPC', 'SPO', 'TWO'])]
    
    hd06: Annotated[D_609, Title(''), Usage('N'), Position(6)]
    
    hd07: Annotated[D_609, Title(''), Usage('N'), Position(7)]
    
    hd08: Annotated[D_1209, Title(''), Usage('N'), Position(8)]
    
    hd09: Annotated[D_1073, Title(''), Usage('S'), Position(9), Enumerated(*['N', 'Y'])]
    
    hd10: Annotated[D_1211, Title(''), Usage('N'), Position(10)]
    
    hd11: Annotated[D_1073, Title(''), Usage('N'), Position(11)]


class L2300_DTP(Segment):
    """"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title(''), Usage('R'), Position(1), Enumerated(*['300', '303', '343', '348', '349', '543', '695'])]
    
    dtp02: Annotated[D_1250, Title(''), Usage('R'), Position(2), Enumerated(*['D8', 'RD8'])]
    
    dtp03: Annotated[D_1251, Title(''), Usage('R'), Position(3)]


class L2300_AMT(Segment):
    """"""
    _segment_name = 'AMT'
    
    amt01: Annotated[D_522, Title(''), Usage('R'), Position(1), Enumerated(*['B9', 'C1', 'D2', 'EBA', 'FK', 'P3', 'R'])]
    
    amt02: Annotated[D_782, Title(''), Usage('R'), Position(2)]
    
    amt03: Annotated[D_478, Title(''), Usage('N'), Position(3)]


class L2300_REF04(Composite):
    """"""
    pass


class L2300_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['17', '1L', '9V', 'CE', 'E8', 'M7', 'PID', 'RB', 'X9', 'XM', 'XX1', 'XX2', 'ZX', 'ZZ'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L2300_REF04(Composite):
    """"""
    pass


class L2300_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['QQ'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L2300_IDC(Segment):
    """"""
    _segment_name = 'IDC'
    
    idc01: Annotated[D_1204, Title(''), Usage('R'), Position(1)]
    
    idc02: Annotated[D_1215, Title(''), Usage('R'), Position(2), Enumerated(*['D', 'H', 'P'])]
    
    idc03: Annotated[D_380, Title(''), Usage('S'), Position(3)]
    
    idc04: Annotated[D_306, Title(''), Usage('S'), Position(4), Enumerated(*['1', '2', 'RX'])]


class L2310_LX(Segment):
    """"""
    _segment_name = 'LX'
    
    lx01: Annotated[D_554, Title(''), Usage('R'), Position(1)]


class L2310_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['1X', '3D', '80', 'FA', 'OD', 'P3', 'QA', 'QN', 'Y2'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['1', '2'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('S'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('S'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('S'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('S'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('S'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('S'), Position(8), Enumerated(*['34', 'FI', 'SV', 'XX'])]
    
    nm109: Annotated[D_67, Title(''), Usage('S'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('R'), Position(10), Enumerated(*['25', '26', '72'])]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2310_N3(Segment):
    """"""
    _segment_name = 'N3'
    
    n301: Annotated[D_166, Title(''), Usage('R'), Position(1)]
    
    n302: Annotated[D_166, Title(''), Usage('S'), Position(2)]


class L2310_N4(Segment):
    """"""
    _segment_name = 'N4'
    
    n401: Annotated[D_19, Title(''), Usage('R'), Position(1)]
    
    n402: Annotated[D_156, Title(''), Usage('S'), Position(2), Enumerated(*states)]
    
    n403: Annotated[D_116, Title(''), Usage('S'), Position(3)]
    
    n404: Annotated[D_26, Title(''), Usage('S'), Position(4), Enumerated(*country)]
    
    n405: Annotated[D_309, Title(''), Usage('N'), Position(5)]
    
    n406: Annotated[D_310, Title(''), Usage('N'), Position(6)]
    
    n407: Annotated[D_1715, Title(''), Usage('S'), Position(7)]


class L2310_PER(Segment):
    """"""
    _segment_name = 'PER'
    
    per01: Annotated[D_366, Title(''), Usage('R'), Position(1), Enumerated(*['IC'])]
    
    per02: Annotated[D_93, Title(''), Usage('N'), Position(2)]
    
    per03: Annotated[D_365, Title(''), Usage('R'), Position(3), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'HP', 'TE', 'WP'])]
    
    per04: Annotated[D_364, Title(''), Usage('R'), Position(4)]
    
    per05: Annotated[D_365, Title(''), Usage('S'), Position(5), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'HP', 'TE', 'WP'])]
    
    per06: Annotated[D_364, Title(''), Usage('S'), Position(6)]
    
    per07: Annotated[D_365, Title(''), Usage('S'), Position(7), Enumerated(*['AP', 'BN', 'CP', 'EM', 'EX', 'FX', 'HP', 'TE', 'WP'])]
    
    per08: Annotated[D_364, Title(''), Usage('S'), Position(8)]
    
    per09: Annotated[D_443, Title(''), Usage('N'), Position(9)]


class L2310_PLA(Segment):
    """"""
    _segment_name = 'PLA'
    
    pla01: Annotated[D_306, Title(''), Usage('R'), Position(1), Enumerated(*['2'])]
    
    pla02: Annotated[D_98, Title(''), Usage('R'), Position(2), Enumerated(*['1P'])]
    
    pla03: Annotated[D_373, Title(''), Usage('R'), Position(3)]
    
    pla04: Annotated[D_337, Title(''), Usage('N'), Position(4)]
    
    pla05: Annotated[D_1203, Title(''), Usage('R'), Position(5), Enumerated(*['14', '22', '46', 'AA', 'AB', 'AC', 'AD', 'AE', 'AF', 'AG', 'AH', 'AI', 'AJ'])]


class L2310(Loop):
    
    lx: Annotated[L2310_LX, Title(''), Usage('R'), Position(3100), Required(True)]
    
    nm1: Annotated[L2310_NM1, Title(''), Usage('R'), Position(3200), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    ItemN3: TypeAlias = Annotated[L2310_N3, Title(''), Usage('S'), Position(3500), Required(True)]
    n3: Annotated[list[ItemN3], MaxItems(2)]
    
    n4: Annotated[L2310_N4, Title(''), Usage('S'), Position(3600), Syntax(['E0207', 'C0605', 'C0704']), Required(True)]
    ItemPer: TypeAlias = Annotated[L2310_PER, Title(''), Usage('S'), Position(3700), Syntax(['P0304', 'P0506', 'P0708']), Required(True)]
    per: Annotated[list[ItemPer], MaxItems(2)]
    
    pla: Annotated[L2310_PLA, Title(''), Usage('S'), Position(3950), Required(True)]


class L2320_COB(Segment):
    """"""
    _segment_name = 'COB'
    
    cob01: Annotated[D_1138, Title(''), Usage('R'), Position(1), Enumerated(*['P', 'S', 'T', 'U'])]
    
    cob02: Annotated[D_127, Title(''), Usage('S'), Position(2)]
    
    cob03: Annotated[D_1143, Title(''), Usage('R'), Position(3), Enumerated(*['1', '5', '6'])]
    
    cob04: Annotated[D_1365, Title(''), Usage('S'), Position(4), Enumerated(*['1', '35', '48', '50', '54', '89', '90', 'A4', 'AG', 'AL', 'BB'])]


class L2320_REF04(Composite):
    """"""
    pass


class L2320_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['60', '6P', 'SY', 'ZZ'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L2320_DTP(Segment):
    """"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title(''), Usage('R'), Position(1), Enumerated(*['344', '345'])]
    
    dtp02: Annotated[D_1250, Title(''), Usage('R'), Position(2), Enumerated(*['D8'])]
    
    dtp03: Annotated[D_1251, Title(''), Usage('R'), Position(3)]


class L2330_NM1(Segment):
    """"""
    _segment_name = 'NM1'
    
    nm101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['36', 'GW', 'IN'])]
    
    nm102: Annotated[D_1065, Title(''), Usage('R'), Position(2), Enumerated(*['2'])]
    
    nm103: Annotated[D_1035, Title(''), Usage('S'), Position(3)]
    
    nm104: Annotated[D_1036, Title(''), Usage('N'), Position(4)]
    
    nm105: Annotated[D_1037, Title(''), Usage('N'), Position(5)]
    
    nm106: Annotated[D_1038, Title(''), Usage('N'), Position(6)]
    
    nm107: Annotated[D_1039, Title(''), Usage('N'), Position(7)]
    
    nm108: Annotated[D_66, Title(''), Usage('S'), Position(8), Enumerated(*['FI', 'NI', 'XV'])]
    
    nm109: Annotated[D_67, Title(''), Usage('S'), Position(9)]
    
    nm110: Annotated[D_706, Title(''), Usage('N'), Position(10)]
    
    nm111: Annotated[D_98, Title(''), Usage('N'), Position(11)]
    
    nm112: Annotated[D_1035, Title(''), Usage('N'), Position(12)]


class L2330_N3(Segment):
    """"""
    _segment_name = 'N3'
    
    n301: Annotated[D_166, Title(''), Usage('R'), Position(1)]
    
    n302: Annotated[D_166, Title(''), Usage('S'), Position(2)]


class L2330_N4(Segment):
    """"""
    _segment_name = 'N4'
    
    n401: Annotated[D_19, Title(''), Usage('R'), Position(1)]
    
    n402: Annotated[D_156, Title(''), Usage('S'), Position(2), Enumerated(*states)]
    
    n403: Annotated[D_116, Title(''), Usage('S'), Position(3)]
    
    n404: Annotated[D_26, Title(''), Usage('S'), Position(4), Enumerated(*country)]
    
    n405: Annotated[D_309, Title(''), Usage('N'), Position(5)]
    
    n406: Annotated[D_310, Title(''), Usage('N'), Position(6)]
    
    n407: Annotated[D_1715, Title(''), Usage('S'), Position(7)]


class L2330_PER(Segment):
    """"""
    _segment_name = 'PER'
    
    per01: Annotated[D_366, Title(''), Usage('R'), Position(1), Enumerated(*['CN'])]
    
    per02: Annotated[D_93, Title(''), Usage('N'), Position(2)]
    
    per03: Annotated[D_365, Title(''), Usage('R'), Position(3), Enumerated(*['TE'])]
    
    per04: Annotated[D_364, Title(''), Usage('R'), Position(4)]
    
    per05: Annotated[D_365, Title(''), Usage('N'), Position(5)]
    
    per06: Annotated[D_364, Title(''), Usage('N'), Position(6)]
    
    per07: Annotated[D_365, Title(''), Usage('N'), Position(7)]
    
    per08: Annotated[D_364, Title(''), Usage('N'), Position(8)]
    
    per09: Annotated[D_443, Title(''), Usage('N'), Position(9)]


class L2330(Loop):
    
    nm1: Annotated[L2330_NM1, Title(''), Usage('R'), Position(4100), Syntax(['P0809', 'C1110', 'C1203']), Required(True)]
    
    n3: Annotated[L2330_N3, Title(''), Usage('S'), Position(4300), Required(True)]
    
    n4: Annotated[L2330_N4, Title(''), Usage('S'), Position(4400), Syntax(['E0207', 'C0605', 'C0704']), Required(True)]
    
    per: Annotated[L2330_PER, Title(''), Usage('S'), Position(4500), Syntax(['P0304', 'P0506', 'P0708']), Required(True)]


class L2320(Loop):
    
    cob: Annotated[L2320_COB, Title(''), Usage('R'), Position(4000), Required(True)]
    ItemRef: TypeAlias = Annotated[L2320_REF, Title(''), Usage('S'), Position(4050), Syntax(['R0203']), Required(True)]
    ref: Annotated[list[ItemRef], MaxItems(4)]
    ItemDtp: TypeAlias = Annotated[L2320_DTP, Title(''), Usage('S'), Position(4070), Required(True)]
    dtp: Annotated[list[ItemDtp], MaxItems(2)]
    ItemL2330: TypeAlias = Annotated[L2330, Title(''), Usage('R'), Position(4510), Required(True)]
    l2330: Annotated[list[ItemL2330], MinItems(3)]


class L2300(Loop):
    
    hd: Annotated[L2300_HD, Title(''), Usage('R'), Position(2600), Required(True)]
    ItemDtp: TypeAlias = Annotated[L2300_DTP, Title(''), Usage('R'), Position(2700), Required(True)]
    dtp: Annotated[list[ItemDtp], MaxItems(6)]
    ItemAmt: TypeAlias = Annotated[L2300_AMT, Title(''), Usage('S'), Position(2800), Required(True)]
    amt: Annotated[list[ItemAmt], MaxItems(9)]
    ItemRef: TypeAlias = Annotated[L2300_REF, Title(''), Usage('S'), Position(2900), Syntax(['R0203']), Required(True)]
    ref: Annotated[list[ItemRef], MaxItems(14)]
    
    ref: Annotated[L2300_REF, Title(''), Usage('S'), Position(2900), Syntax(['R0203']), Required(True)]
    ItemIdc: TypeAlias = Annotated[L2300_IDC, Title(''), Usage('S'), Position(3000), Required(True)]
    idc: Annotated[list[ItemIdc], MaxItems(3)]
    ItemL2310: TypeAlias = Annotated[L2310, Title(''), Usage('R'), Position(3100), Required(True)]
    l2310: Annotated[list[ItemL2310], MinItems(30)]
    ItemL2320: TypeAlias = Annotated[L2320, Title(''), Usage('R'), Position(4000), Required(True)]
    l2320: Annotated[list[ItemL2320], MinItems(5)]


class L2700_LS_LS(Segment):
    """"""
    _segment_name = 'LS'
    
    ls01: Annotated[D_447, Title(''), Usage('R'), Position(1), Enumerated(*['2700'])]


class L2700_LX(Segment):
    """"""
    _segment_name = 'LX'
    
    lx01: Annotated[D_554, Title(''), Usage('R'), Position(1)]


class L2750_N1(Segment):
    """"""
    _segment_name = 'N1'
    
    n101: Annotated[D_98, Title(''), Usage('R'), Position(1), Enumerated(*['75'])]
    
    n102: Annotated[D_93, Title(''), Usage('R'), Position(2)]
    
    n103: Annotated[D_66, Title(''), Usage('N'), Position(3)]
    
    n104: Annotated[D_67, Title(''), Usage('N'), Position(4)]
    
    n105: Annotated[D_706, Title(''), Usage('N'), Position(5)]
    
    n106: Annotated[D_98, Title(''), Usage('N'), Position(6)]


class L2750_REF04(Composite):
    """"""
    pass


class L2750_REF(Segment):
    """"""
    _segment_name = 'REF'
    
    ref01: Annotated[D_128, Title(''), Usage('R'), Position(1), Enumerated(*['00', '17', '18', '19', '26', '3L', '6M', '9V', '9X', 'GE', 'LU', 'PID', 'XX1', 'XX2', 'YY', 'ZZ'])]
    
    ref02: Annotated[D_127, Title(''), Usage('R'), Position(2)]
    
    ref03: Annotated[D_352, Title(''), Usage('N'), Position(3)]


class L2750_DTP(Segment):
    """"""
    _segment_name = 'DTP'
    
    dtp01: Annotated[D_374, Title(''), Usage('R'), Position(1), Enumerated(*['007'])]
    
    dtp02: Annotated[D_1250, Title(''), Usage('R'), Position(2), Enumerated(*['D8', 'RD8'])]
    
    dtp03: Annotated[D_1251, Title(''), Usage('R'), Position(3)]


class L2750(Loop):
    
    n1: Annotated[L2750_N1, Title(''), Usage('R'), Position(6882), Syntax(['R0203', 'P0304']), Required(True)]
    
    ref: Annotated[L2750_REF, Title(''), Usage('S'), Position(6883), Syntax(['R0203']), Required(True)]
    
    dtp: Annotated[L2750_DTP, Title(''), Usage('S'), Position(6884), Required(True)]


class L2700_LE(Segment):
    """"""
    _segment_name = 'LE'
    
    le01: Annotated[D_447, Title(''), Usage('R'), Position(1)]


class L2700(Loop):
    
    lx: Annotated[L2700_LX, Title(''), Usage('R'), Position(6881), Required(True)]
    ItemL2750: TypeAlias = Annotated[L2750, Title(''), Usage('R'), Position(6882), Required(True)]
    l2750: Annotated[list[ItemL2750], MinItems(1)]
    
    le: Annotated[L2700_LE, Title(''), Usage('S'), Position(6885), Required(True)]


class L2700_LS(Loop):
    
    ls: Annotated[L2700_LS_LS, Title(''), Usage('R'), Position(6880), Required(True)]
    ItemL2700: TypeAlias = Annotated[L2700, Title(''), Usage('R'), Position(6881), Required(True)]
    l2700: Annotated[list[ItemL2700], MinItems(1)]


class L2000(Loop):
    
    ins: Annotated[L2000_INS, Title(''), Usage('R'), Position(100), Syntax(['P1112']), Required(True)]
    
    ref: Annotated[L2000_REF, Title(''), Usage('R'), Position(200), Syntax(['R0203']), Required(True)]
    
    ref: Annotated[L2000_REF, Title(''), Usage('S'), Position(200), Syntax(['R0203']), Required(True)]
    ItemRef: TypeAlias = Annotated[L2000_REF, Title(''), Usage('S'), Position(200), Syntax(['R0203']), Required(True)]
    ref: Annotated[list[ItemRef], MaxItems(13)]
    ItemDtp: TypeAlias = Annotated[L2000_DTP, Title(''), Usage('S'), Position(250), Required(True)]
    dtp: Annotated[list[ItemDtp], MaxItems(24)]
    ItemL2100A: TypeAlias = Annotated[L2100A, Title(''), Usage('R'), Position(300), Required(True)]
    l2100a: Annotated[list[ItemL2100A], MinItems(1)]
    ItemL2100B: TypeAlias = Annotated[L2100B, Title(''), Usage('R'), Position(300), Required(True)]
    l2100b: Annotated[list[ItemL2100B], MinItems(1)]
    ItemL2100C: TypeAlias = Annotated[L2100C, Title(''), Usage('R'), Position(300), Required(True)]
    l2100c: Annotated[list[ItemL2100C], MinItems(1)]
    ItemL2100D: TypeAlias = Annotated[L2100D, Title(''), Usage('R'), Position(300), Required(True)]
    l2100d: Annotated[list[ItemL2100D], MinItems(3)]
    ItemL2100E: TypeAlias = Annotated[L2100E, Title(''), Usage('R'), Position(300), Required(True)]
    l2100e: Annotated[list[ItemL2100E], MinItems(3)]
    ItemL2100F: TypeAlias = Annotated[L2100F, Title(''), Usage('R'), Position(300), Required(True)]
    l2100f: Annotated[list[ItemL2100F], MinItems(1)]
    ItemL2100G: TypeAlias = Annotated[L2100G, Title(''), Usage('R'), Position(300), Required(True)]
    l2100g: Annotated[list[ItemL2100G], MinItems(13)]
    ItemL2100H: TypeAlias = Annotated[L2100H, Title(''), Usage('R'), Position(700), Required(True)]
    l2100h: Annotated[list[ItemL2100H], MinItems(1)]
    ItemL2200: TypeAlias = Annotated[L2200, Title(''), Usage('R'), Position(2000), Required(True)]
    l2200: Annotated[list[ItemL2200], MinItems(1)]
    ItemL2300: TypeAlias = Annotated[L2300, Title(''), Usage('R'), Position(2600), Required(True)]
    l2300: Annotated[list[ItemL2300], MinItems(99)]
    ItemL2700_Ls: TypeAlias = Annotated[L2700_LS, Title(''), Usage('S'), Position(6800), Required(True)]
    l2700_ls: Annotated[list[ItemL2700_Ls], MinItems(1)]


class TABLE2AREA2(Loop):
    ItemL2000: TypeAlias = Annotated[L2000, Title(''), Usage('R'), Position(100), Required(True)]
    l2000: Annotated[list[ItemL2000], MinItems(1)]


class ST_LOOP_SE(Segment):
    """"""
    _segment_name = 'SE'
    
    se01: Annotated[D_96, Title(''), Usage('R'), Position(1)]
    
    se02: Annotated[D_329, Title(''), Usage('R'), Position(2)]


class ST_LOOP(Loop):
    
    st: Annotated[ST_LOOP_ST, Title(''), Usage('R'), Position(100), Required(True)]
    ItemHeader: TypeAlias = Annotated[HEADER, Title(''), Usage('R'), Position(110), Required(True)]
    header: Annotated[list[ItemHeader], MinItems(1)]
    ItemTable2Area2: TypeAlias = Annotated[TABLE2AREA2, Title(''), Usage('R'), Position(120), Required(True)]
    table2area2: Annotated[list[ItemTable2Area2], MinItems(1)]
    
    se: Annotated[ST_LOOP_SE, Title(''), Usage('R'), Position(6900), Required(True)]


class MSG834A1(Message):
    """HIPAA Benefit Enrollment and Maintenance 005010X220A1 834A1"""
    ItemSt_Loop: TypeAlias = Annotated[ST_LOOP, Title(''), Usage('R'), Position(200), Required(True)]
    st_loop: Annotated[list[ItemSt_Loop], MinItems(1)]



"""
Test tools/xml_extract

..  todo:: See below in test_xml_convert -- use a visitor to examine the AST.
"""
import ast
from io import StringIO
from typing import Annotated
from types import UnionType

from pytest import fixture, CaptureFixture, mark
from unittest.mock import sentinel, Mock, call

from tools import xml_extract
from x12 import base


def test_codeset() -> None:
    c = xml_extract.Codeset(sentinel.ID, sentinel.DATA_ELE, sentinel.SOURCE, [sentinel.VALUE])
    assert c.label == sentinel.ID
    assert c.jsonschema() == [sentinel.VALUE]

@fixture
def mock_codes_xml() -> StringIO:
    mock_doc = StringIO("""<?xml version="1.0" encoding="UTF-8"?>
    <codesets xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
      <codeset>
        <id>states</id>
        <name>US State or Province Codes</name>
        <data_ele>156</data_ele>
        <version>
          <code>NC</code>
        </version>
      </codeset>
    </codesets>
    """)
    return mock_doc

def test_code_reader(mock_codes_xml: StringIO) -> None:
    codes = list(xml_extract.code_reader(mock_codes_xml))
    assert len(codes) == 1
    assert codes[0].id == "states"
    assert codes[0].data_ele == "156"
    assert codes[0].values == ["NC"]

def test_dataelement() -> None:
    d = xml_extract.DataElement('42', "AN", "3", "5", sentinel.NAME)
    assert d.jsonschema() == {
        "title": sentinel.NAME,
        "type": "string",
        "data_type_code": "AN",
        "minLength": 3,
        "maxLength": 5,
    }
    assert d.annotation() == ('D_42: TypeAlias = Annotated[str, Title(sentinel.NAME), MinLen(3), MaxLen(5)]', "")

@fixture
def mock_elements_xml() -> StringIO:
    mock_doc = StringIO("""<?xml version="1.0" encoding="UTF-8"?>
    <data_elements>
    <data_ele ele_num="I01" data_type="ID" min_len="2" max_len="2" name="I01"/>
    </data_elements>
    """)
    return mock_doc

def test_element_reader(mock_elements_xml: StringIO) -> None:
    elements = list(xml_extract.element_reader(mock_elements_xml))
    assert len(elements) == 1
    assert elements[0].ele_num == "I01"
    assert elements[0].data_type == "ID"
    assert elements[0].min_len == "2"
    assert elements[0].min_len == "2"
    assert elements[0].name == "I01"

def test_schemamap() -> None:
    s = xml_extract.SchemaMap(sentinel.ICVN, sentinel.VRIIC, sentinel.FIC, sentinel.ABBR, sentinel.DEFINITION)
    assert s.icvn == sentinel.ICVN
    assert s.vriic == sentinel.VRIIC
    assert s.fic == sentinel.FIC
    assert s.abbr == sentinel.ABBR
    assert s.definition == sentinel.DEFINITION

@fixture
def mock_mapping() -> str:
    mock_doc = StringIO("""<?xml version="1.0" encoding="UTF-8"?>
    <maps>
      <version icvn="00400">
        <map vriic="004010" fic="PS" abbr="830">830.4010.PS.xml</map>
      </version>
    </maps>
    """)
    return mock_doc

def test_map_reader(mock_mapping) -> None:
    maps = list(xml_extract.map_reader(mock_mapping))
    assert len(maps) == 1
    assert maps[0].icvn == "00400"
    assert maps[0].vriic == "004010"
    assert maps[0].fic == "PS"
    assert maps[0].abbr == "830"
    assert maps[0].definition == "830.4010.PS.xml"

def test_pythonify() -> None:
    assert xml_extract.pythonify("ISA", "X") == "ISA"
    assert xml_extract.pythonify("ISA01-02", "X") == "ISA01_02"
    assert xml_extract.pythonify("ISA01.02", "X") == "ISA01_02"
    assert xml_extract.pythonify("123", "D") == "D123"

def test_element() -> None:
    e = xml_extract.Element(
        xid="XID-E.1",
        data_ele=sentinel.DATA_ELE,
        name=sentinel.NAME,
        usage=sentinel.USAGE,
        seq="42",
        refdes=sentinel.REFDES,
        repeat="1",
        codes=[sentinel.CODES],
        external=sentinel.EXTERNAL,
        common_def=sentinel.COMMON_DEF,
        loop=sentinel.LOOP
    )
    assert e.attr_name == "XID_E_1".lower()
    assert e.class_name == "sentinel.LOOP_XID_E_1"
    assert e.jsonschema() == {
        'items': {
            'description': 'xid=XID-E.1 data_ele=sentinel.DATA_ELE',
            'position': 42,
            'title': sentinel.NAME,
            'type': {'allOf': [{'$ref': '#/$common/sentinel.COMMON_DEF'},
                              {'enum': [sentinel.CODES]}]},
            'usage': sentinel.USAGE},
         'maxItems': 1,
         'type': 'array'} != {'description': 'xid=XID-E.1 data_ele=sentinel.DATA_ELE',
         'maxItems': 1,
         'position': 42,
         'title': sentinel.NAME,
         'type': 'array',
         'usage': sentinel.USAGE}

def test_composite() -> None:
    e = Mock(
            name="mock element",
            attr_name=sentinel.ELEMENT,
            usage="R",
            jsonschema=Mock(return_value={"schema": None})
    )
    c = xml_extract.Composite(
        xid="XID-C.1",
        data_ele=sentinel.DATA_ELE,
        name=sentinel.NAME,
        usage=sentinel.USAGE,
        seq="42",
        refdes=sentinel.REFDES,
        syntax=[sentinel.SYNTAX],
        repeat="1",
        elements=[e],
        loop=sentinel.LOOP
    )
    assert c.attr_name == "XID_C_1".lower()
    assert c.class_name == "sentinel.LOOP_XID_C_1"
    assert c.required
    assert c.jsonschema() == {
        'items': {
            'description': 'xid=XID-C.1 name=sentinel.NAME refdes=sentinel.REFDES data_ele=sentinel.DATA_ELE',
            'properties': {sentinel.ELEMENT: {"schema": None}},
            'required': [sentinel.ELEMENT],
            'sequence': 42,
            'syntax': [sentinel.SYNTAX],
            'title': sentinel.NAME,
            'type': 'object',
            'usage': sentinel.USAGE},
        'maxItems': 1,
        'type': 'array'}


def test_segment() -> None:
    e = Mock(
        name="mock element",
        attr_name=sentinel.ELEMENT,
        usage="R",
        jsonschema=Mock(return_value={"schema": None}),
        class_name="Element"
    )
    c = Mock(
        name="mock composite",
        attr_name=sentinel.COMPOSITE,
        usage="R",
        jsonschema=Mock(return_value={"schema": None}),
        class_name = "Composite"
    )

    s = xml_extract.Segment(
        xid="XID-S.1",
        name=sentinel.NAME,
        end_tag=sentinel.END_TAG,
        usage=sentinel.USAGE,
        pos="42",
        max_use=">1",
        syntax=[sentinel.SYNTAX],
        children=[c, e],
        loop=sentinel.LOOP
    )
    assert s.attr_name == "XID_S_1".lower()
    assert s.class_name == "sentinel.LOOP_XID_S_1"
    assert s.required == [sentinel.COMPOSITE, sentinel.ELEMENT]
    assert s.repeat
    assert s.jsonschema() == {
        'items': {'description': 'xid=XID-S.1 name=sentinel.NAME',
            'endTag': sentinel.END_TAG,
            'position': 42,
            'properties': {'xid': {'literal': 'XID-S.1'},
                          sentinel.COMPOSITE: {'$ref': '#/$elements/Composite'},
                          sentinel.ELEMENT: {'$ref': '#/$elements/Element'}},
            'required': [sentinel.COMPOSITE, sentinel.ELEMENT],
            'syntax': [sentinel.SYNTAX],
            'title': sentinel.NAME,
            'type': 'object',
            'usage': sentinel.USAGE},
        'type': 'array'}

def test_loop() -> None:
    s = Mock(
        name="mock segmnent",
        attr_name=sentinel.SEGMENT,
        usage="R",
        jsonschema=Mock(return_value={"schema": None}),
        class_name = "Segment"
    )
    l = xml_extract.Loop(
        xid="XID-L.1",
        name=sentinel.NAME,
        usage=sentinel.USAGE,
        pos="42",
        repeat="1",
        type=sentinel.TYPE,
        children=[s],
    )
    assert l.attr_name == "XID_L_1".lower()
    assert l.class_name == "XID_L_1"
    assert l.required == [sentinel.SEGMENT]
    assert l.repeat
    assert l.jsonschema() == {
        'items': {'description': 'xid=XID-L.1 name=sentinel.NAME type=sentinel.TYPE',
               'position': 42,
               'properties': {sentinel.SEGMENT: {'$ref': '#/$segments/Segment'}},
               'required': [sentinel.SEGMENT],
               'title': sentinel.NAME,
               'type': 'object',
               'usage': sentinel.USAGE},
        'maxItems': 1,
        'type': 'array'}

def test_message() -> None:
    m = xml_extract.Message(
        xid="999",
        name=sentinel.NAME,
        loops=[Mock(name="mock loop", usage="R", attr_name=sentinel.LOOP, class_name=sentinel.LOOP_CLASS)]
    )

    assert m.class_name == "MSG999"
    assert m.required == [sentinel.LOOP]
    assert m.jsonschema() == {
        'description': 'xid=999 name=sentinel.NAME',
        'properties': {sentinel.LOOP: {'$ref': "#/$loops/sentinel.LOOP_CLASS"}},
        'required': [sentinel.LOOP],
        'title': sentinel.NAME,
        'type': 'object'}

@fixture
def mock_message_xml() -> str:
    mock_doc = StringIO("""<?xml version="1.0" encoding="UTF-8"?>
    <transaction xid="999">
        <name>Test Case 999</name>
        <loop xid="ISA_LOOP" type="explicit">
            <name>Interchange Control Header</name>
            <usage>R</usage>
            <pos>001</pos>
            <repeat>&gt;1</repeat>
            <segment xid="ISA">
              <name>Interchange Control Header</name>
              <usage>R</usage>
              <pos>010</pos>
              <max_use>1</max_use>
              <element xid="ISA01">
                <data_ele>I01</data_ele>
                <name>Authorization Information Qualifier</name>
                <usage>R</usage>
                <seq>01</seq>
                <valid_codes>
                  <code>00</code>
                  <code>03</code>
                </valid_codes>
              </element>
          </segment>
        </loop>
    </transaction>
    """)
    return mock_doc


def test_message_reader(mock_message_xml) -> None:
    m = xml_extract.message_reader(mock_message_xml)
    assert m.xid == "999"
    assert m.name == "Test Case 999"
    assert len(m.loops) == 1

    loop = m.loops[0]
    assert loop.xid == "ISA_LOOP"
    assert loop.name == "Interchange Control Header"
    assert loop.usage == "R"
    assert loop.pos == "001"
    assert loop.repeat == ">1"
    assert len(loop.children) == 1

    segment = loop.children[0]
    assert segment.xid == "ISA"
    assert segment.name == "Interchange Control Header"
    assert segment.usage == "R"
    assert segment.pos == "010"
    assert segment.max_use == "1"
    assert len(segment.children) == 1

    element = segment.children[0]
    assert element.xid == "ISA01"
    assert element.data_ele == "I01"
    assert element.name == "Authorization Information Qualifier"
    assert element.usage == "R"
    assert element.seq == "01"
    assert element.common_def == "I01"
    assert element.codes == ["00", "03"]

@fixture
def mock_message() -> xml_extract.Message:
    e = xml_extract.Element(
        xid="ISA01",
        data_ele="I01",
        name="Authorization Information Qualifier",
        usage="R",
        seq="01",
        refdes="",
        repeat="",
        codes=["00", "03"],
        external="",
        common_def="I01",
        loop="ISA_LOOP"
    )
    s = xml_extract.Segment(
        xid="ISA",
        name="Interchange Control Header",
        end_tag="IEA",
        usage="R",
        pos="010",
        max_use="1",
        syntax=[],
        children=[e],
        loop="ISA_LOOP"
    )
    l = xml_extract.Loop(
        xid="ISA_LOOP",
        name="ISA LOOP",
        usage="R",
        pos="001",
        repeat=">1",
        type="explicit",
        children=[s]
    )
    m = xml_extract.Message(
        xid="999",
        name="Test Case 999",
        loops=[l]
    )
    return m


class ClassVisitor(ast.NodeVisitor):
    def __init__(self) -> None:
        self.names = []
    def visit_ClassDef(self, node: ast.ClassDef) -> None:
        # print(node.body)
        # TODO: Visit the body. They're all very similar.
        # optional ast.Expr (the docstring)
        # internal ast.ClassDef (name == "Schema")
        # the fields are the remaining ast.AnnAssign statments
        self.names.append(node.name)

def test_emit_python_interim(mock_message: xml_extract.Message, capsys: CaptureFixture) -> None:
    data_elements = {"I01": Mock(name="Mock DataElement", min_len="2", max_len="2", int_min_len=2, int_max_len=2)}
    codes = {}
    ep = xml_extract.EmitPython_Interim().data_elements(data_elements).codesets(codes)
    ep.visit(mock_message)
    python_source, _ = capsys.readouterr()

    # Assert the code is valid
    module = ast.parse(python_source, filename="__test__", mode="exec")

    # TODO: Check the structure of the code more closely
    cv = ClassVisitor()
    cv.visit(module)
    assert cv.names == ['ISA_LOOP_ISA01', 'ISA_LOOP_ISA', 'ISA_LOOP', 'MSG999']

def test_emit_python_annotated_codesets(mock_message: xml_extract.Message, capsys: CaptureFixture) -> None:
    data_elements = {"I01": Mock(name="Mock DataElement", min_len="2", max_len="2", int_min_len=2, int_max_len=2)}
    codes = {'CS': Mock(name="mock_Codeset", values=["4", "2"])}
    ep = xml_extract.EmitPython_Annotated().data_elements(data_elements).codesets(codes)

    ep.dump_codesets()
    python_source, _ = capsys.readouterr()
    # print(python_source.splitlines())
    assert python_source.splitlines() == [
        "CS = ['4', '2']"
    ]

def test_emit_python_annotated_data_elements(mock_message: xml_extract.Message, capsys: CaptureFixture) -> None:
    data_elements = {"I01": Mock(name="Mock DataElement", min_len="2", max_len="2", int_min_len=2, int_max_len=2, data_type="N2")}
    codes = {}
    ep = xml_extract.EmitPython_Annotated().data_elements(data_elements).codesets(codes)

    ep.dump_data_elements()
    python_source, _ = capsys.readouterr()
    # print(python_source.splitlines())
    assert python_source.splitlines() == [
        "AN: TypeAlias = str",
        "B: TypeAlias = str",
        "DT: TypeAlias = datetime.date",
        'ID: TypeAlias = str',
        "R: TypeAlias = float",
        "TM: TypeAlias = datetime.time",
        "N: TypeAlias = int",
        'N0: TypeAlias = Annotated[Decimal, Scale(0)]',
        'N1: TypeAlias = Annotated[Decimal, Scale(1)]',
        'N2: TypeAlias = Annotated[Decimal, Scale(2)]',
        'N3: TypeAlias = Annotated[Decimal, Scale(3)]',
        'N4: TypeAlias = Annotated[Decimal, Scale(4)]',
        'N5: TypeAlias = Annotated[Decimal, Scale(5)]',
        'N6: TypeAlias = Annotated[Decimal, Scale(6)]',
        'N7: TypeAlias = Annotated[Decimal, Scale(7)]',
        'N8: TypeAlias = Annotated[Decimal, Scale(8)]',
        'N9: TypeAlias = Annotated[Decimal, Scale(9)]',
        "I01: TypeAlias = Annotated[N2, MinLen(2), MaxLen(2)]"
    ]


def test_emit_python_annotated_message(mock_message: xml_extract.Message, capsys: CaptureFixture) -> None:
    data_elements = {"I01": Mock(name="Mock DataElement", min_len="2", max_len="2", int_min_len=2, int_max_len=2)}
    codes = {}
    ep = xml_extract.EmitPython_Annotated().data_elements(data_elements).codesets(codes)

    ep.visit(mock_message)
    python_source, _ = capsys.readouterr()

    # Assert the code is valid
    module_ast = ast.parse(python_source, filename="__test__", mode="exec")

    # TODO: Check the structure of the code more closely.
    cv = ClassVisitor()
    cv.visit(module_ast)
    assert cv.names == ['ISA_LOOP_ISA', 'ISA_LOOP', 'MSG999']

    # Check the Usability of the generated code.
    module_locals = {}
    exec("from x12.base import *", module_locals, module_locals)
    # TODO: Should create temp common.py module from `data_elements`
    exec("from x12.common import *", module_locals, module_locals)
    # print(module_locals)

    exec(python_source, module_locals, module_locals)
    p = base.X12Parser(module_locals['MSG999'])
    src = base.Source("ISA|00~\n", element_sep="|", array_sep=":", segment_sep="~")
    msg = p.parse(src)
    assert list(msg.segment_iter()) == [['ISA', '00']]



"""
Candidate "main" application to convert Exchange format to JSON and JSON to Exchange format.
"""
import argparse
import importlib
import logging
from pathlib import Path
import sys

from x12.base import Source, Message
# from x12 import msg_834_5010_X220_A1
import x12

logger = logging.getLogger("tigershark3")

def get_options(argv: list[str]) -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument("--format", "-f", choices=("json", "exchange", "segments", "python"), default="json")
    parser.add_argument("--output", "-o", type=Path, default=None)
    parser.add_argument("source", nargs=1, type=Path)
    parser.add_argument("--package", "-p", type=str, default="x12", help="Python package with message modules")
    parser.add_argument("--message", "-m", type=str, default="msg_834_5010_X220_A1.MSG834A1", help="message module.class")
    parser.add_argument("--separators", "-s", type=str, default="", help="separators when parsing compressed ISA segments, for example '*:~'")
    parser.add_argument("--debug", "-d", dest="log_level", action='store_const', const=logging.DEBUG, default=logging.INFO)
    options = parser.parse_args(argv)
    module_name, _, cls_name = options.message.partition(".")
    try:
        importlib.import_module(f"{options.package}.{module_name}", package='x12')
        options.message_class = eval(f"{options.package}.{module_name}.{cls_name}")
    except NameError as ex:
        parser.error(f"unknown message class {options.package}.{options.message}")
    if options.separators:
        try:
            options.element_sep, _, options.segment_sep = options.separators
        except ValueError as ex:
            parser.error(f"separators must be three characters, for example '*:~'")
    else:
        options.element_sep = options.segment_sep = ""
    return options

def exchange_to_python(source: Path, message_class: type[Message], element_sep: str = "", segment_sep: str = "") -> None:
    """
    Output is "segments" format, used for unit test output...
    """
    document = Source(source.read_text(), element_sep=element_sep, segment_sep=segment_sep)
    msg = message_class.parse(document)
    if msg:
        print(repr(msg))
    else:
        logger.error("No message parsed from %s as %r", source, message_class)


def exchange_to_segments(source: Path, message_class: type[Message], element_sep: str = "", segment_sep: str = "") -> None:
    """
    Output is "segments" format, used for unit test output...
    """
    document = Source(source.read_text(), element_sep=element_sep, segment_sep=segment_sep)
    msg = message_class.parse(document)
    if msg:
        print(list(msg.segment_iter()))
    else:
        logger.error("No message parsed from %s as %r", source, message_class)

def exchange_to_json(source: Path, message_class: type[Message], element_sep: str = "", segment_sep: str = "") -> None:
    """
    TODO: Output is "json" format.
    """
    document = Source(source.read_text(), element_sep=element_sep, segment_sep=segment_sep)
    msg = message_class.parse(document)
    print(msg.json())

def main(argv: list[str] = sys.argv[1:]) -> None:
    options = get_options(argv)
    logging.getLogger().setLevel(options.log_level)
    # Assume format is "segments" which lists the segments
    # Assume output is not specified (which means stdout)
    logger.info("Loading %r using %r", options.source[0], options.message_class)
    if options.element_sep or options.segment_sep:
        logger.info("  Compressed ISA, separators %r %r", options.element_sep, options.segment_sep)
    if options.format == "segments":
        exchange_to_segments(options.source[0], options.message_class, options.element_sep, options.segment_sep)
    elif options.format == "python":
        exchange_to_python(options.source[0], options.message_class, options.element_sep, options.segment_sep)
    elif options.format == "json":
        exchange_to_json(options.source[0], options.message_class, options.element_sep, options.segment_sep)
    else:
        raise NotImplementedError(f"unsupporeted {options.format}")

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    message_options = []
    examples = Path.cwd().parent / "tests"

    # message_path = examples / "834-example.txt"
    # message_class = "msg_834_5010_X220_A1.MSG834A1"

    # message_path = examples / "835-example.txt"
    # Failed: message_class = "msg_835_4010_X091_A1.MSG835"  # Unexpected L2100_NM109
    # message_class = "msg_835_5010_X221_A1.MSG835W1"

    # message_path = examples / "835-example-2.txt"
    # message_class = "msg_835_5010_X221_A1.MSG835W1"

    # Some problems with example file and message definitions.
    # message_path = examples / "837I-Examples.txt" # Damanged file
    # message_class = "msg_837Q3_I_5010_X223_A1_v2.MSG837Q3"  # Appears incomplete
    # message_class = "msg_837_5010_X222_A1.MSG837"

    # Multi-transaction files...
    # message_path = examples / "TEST 276 TXNs.txt"
    # message_class = "msg_276_4010_X093_A1.MSG276"
    # message_options = ["-s", "*:~", "-f"]

    # Documentation example.
    # message_path = examples / "271-example.txt"
    # message_class = "msg_271_4010_X092_A1.MSG271"

    # Separators Issues.
    message_path = examples / "TEST 278_13 TXNS.txt"
    message_class = "msg_278_4010_X094_A1.MSG278"

    # Command-line.
    main(["-d", "-m", message_class, "-f", "json"] + message_options + [str(message_path)])
    logging.shutdown()
